@@ -1,9 +1,9 @@
 package Test::Smoke::Reporter;
 use strict;
 
-# $Id: Reporter.pm 1120 2007-09-30 10:37:12Z abeltje $
+# $Id: Reporter.pm 1155 2008-01-03 13:32:28Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.029';
+$VERSION = '0.032';
 
 use Cwd;
 use File::Spec::Functions;
@@ -15,19 +15,22 @@ use Test::Smoke::Util qw( grepccmsg get_smoked_Config
                           time_in_hhmm get_local_patches );
 
 my %CONFIG = (
-    df_ddir       => curdir(),
-    df_outfile    => 'mktest.out',
-    df_rptfile    => 'mktest.rpt',
-    df_cfg        => undef,
-    df_lfile      => undef,
-    df_showcfg    => 0,
+    df_ddir         => curdir(),
+    df_outfile      => 'mktest.out',
+    df_rptfile      => 'mktest.rpt',
+    df_cfg          => undef,
+    df_lfile        => undef,
+    df_showcfg      => 0,
 
-    df_locale     => undef,
-    df_defaultenv => undef,
-    df_is56x      => undef,
-    df_skip_tests => undef,
+    df_locale       => undef,
+    df_defaultenv   => undef,
+    df_is56x        => undef,
+    df_skip_tests   => undef,
 
-    df_v          => 0,
+    df_harnessonly  => undef,
+    df_harness3opts => undef,
+
+    df_v            => 0,
 );
 
 =head1 NAME
@@ -301,6 +304,11 @@ sub _parse {
                 if ref $rpt{$cfgarg}->{$debug}{$tstenv};
             next;
         }
+        if ( /^\s+(?:Bad plan)|(?:No plan found)/ ) {
+            push @{ $rpt{$cfgarg}->{$debug}{$tstenv} }, $_
+                if ref $rpt{$cfgarg}->{$debug}{$tstenv};
+            next;
+        }
         next;
     }
 
@@ -323,7 +331,7 @@ sub _post_process {
 
     unless ( defined $self->{is56x} ) {
         my %cfg = get_smoked_Config( $self->{ddir}, 'version' );
-        my $p_version = sprintf "%d.%03d%03d", split /\./, $cfg{version};
+        my $p_version = sprintf "%d.%03d%03d", split m/\./, $cfg{version};
         $self->{is56x} = $p_version < 5.007;
     }
     $self->{defaultenv} ||= $self->{is56x};
@@ -496,6 +504,8 @@ sub report {
 
     $report .= $self->registered_patches;
 
+    $report .= $self->harness3_options;
+
     $report .= $self->user_skipped_tests;
 
     $report .= "\nFailures: (common-args) $self->{_rpt}{common_args}\n"
@@ -548,7 +558,43 @@ sub registered_patches {
     @lpatches or return "";
 
     my $list = join "\n", map "    $_" => @lpatches;
-    return "Locally applied patches:\n$list\n";
+    return "\nLocally applied patches:\n$list\n";
+}
+
+=item $reporter->harness3_options
+
+Show indication of the options used for C<HARNESS_OPTIONS>.
+
+=cut
+
+sub harness3_options {
+    my $self = shift;
+
+    $self->{harnessonly} or return "";
+
+    my $msg = "\nTestsuite was run only with 'harness'";
+    $self->{harness3opts} or return $msg . "\n";
+
+    return  $msg . " and HARNESS_OPTIONS=$self->{harness3opts}\n";
+}
+
+=item $reporter->user_skipped_tests( )
+
+Show indication for the fact that the user requested to skip some tests.
+
+=cut
+
+sub user_skipped_tests {
+    my( $self ) = @_;
+    $self->{skip_tests} && -f $self->{skip_tests} or return "";
+
+    local *NOTESTS;
+    open NOTESTS, "< $self->{skip_tests}" or return "";
+
+    my $skipped = join "\n", map { chomp; "    $_" } <NOTESTS>;
+    close NOTESTS;
+
+    return "\nTests skipped on user request:\n$skipped";
 }
 
 =item $reporter->ccmessages( )
@@ -664,23 +710,6 @@ sub summary {
     return "Summary: $rpt_summary\n";
 }
 
-=item $reporter->user_skipped_tests( )
-
-=cut
-
-sub user_skipped_tests {
-    my( $self ) = @_;
-    $self->{skip_tests} && -f $self->{skip_tests} or return "";
-
-    local *NOTESTS;
-    open NOTESTS, "< $self->{skip_tests}" or return "";
-
-    my $skipped = join "\n", map { chomp; "    $_" } <NOTESTS>;
-    close NOTESTS;
-
-    return "\nTests skipped on user request:\n$skipped";
-}
-
 =item $repoarter->has_test_failures( )
 
 Returns true if C<< @{ $reporter->{_failures} >>.
@@ -752,9 +781,9 @@ sub bldenv_legend {
                 my $locale = shift @locale;     # XXX: perhaps pop()
                 $line .= "LC_ALL = $locale"
             } else {
-                $line .= ( $i % 2 == 0 )
+                $line .= ( (($i - @{$self->{_locale}}) % $half) % 2 == 0 )
                     ? "PERLIO = perlio"
-                    : "PERLIO = stdio";
+                    : "PERLIO = stdio ";
             }
             $i < $half and $line .= " $debugging";
             $line .= "\n";
