@@ -1,9 +1,9 @@
 package Test::Smoke::Smoker;
 use strict;
 
-# $Id: Smoker.pm 1120 2007-09-30 10:37:12Z abeltje $
+# $Id: Smoker.pm 1155 2008-01-03 13:32:28Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.036';
+$VERSION = '0.038';
 
 use Cwd;
 use File::Spec::Functions qw( :DEFAULT abs2rel rel2abs );
@@ -26,6 +26,8 @@ my %CONFIG = (
     df_is_vms         => $^O eq 'VMS',
     df_vmsmake        => 'MMK',
     df_harnessonly    => scalar ($^O =~ /VMS/),
+    df_hasharness3    => 0,
+    df_harness3opts   => '',
 
     df_is_win32       => $^O eq 'MSWin32',
     df_w32cc          => 'MSVC60',
@@ -49,6 +51,16 @@ sub HARNESS_RE1 () {
 }
 sub HARNESS_RE2() { '^\s+(\d+(?:[-\s]+\d+)*-?)$' }
 
+sub HARNESS3_RE() {
+     '^(?:
+          (?:\ \ Failed\ tests?(?:\ number\(s\))?:\ \ )
+          |
+          \s+
+       )
+       (\d[0-9, -]*)'
+}
+
+
 =head1 NAME
 
 Test::Smoke::Smoker - OO interface to do one smoke cycle.
@@ -494,6 +506,9 @@ sub make_test {
 
 	if  ( $self->{harnessonly} ) {
 
+            $self->{harness3opts} and
+                local $ENV{HARNESS_OPTIONS} = $self->{harness3opts};
+
             $self->make_test_harness( $config );
 
         } else {
@@ -502,9 +517,10 @@ sub make_test {
 
             # MSWin32 builds from its own directory
             if ( $self->{is_win32} ) {
-                $config->has_arg( '-Uuseshrplib' )
-                    and $test_target = 'static-test';
-                $self->_run_harness_target( $test_target );
+#                $config->has_arg( '-Uuseshrplib' )
+#                    and $test_target = 'static-test';
+#                $self->_run_harness_target( $test_target );
+                $self->make_test_harness( $config );
             } else {
                 $self->_run_TEST_target( $test_target, 1 );
             }
@@ -547,8 +563,8 @@ sub extend_with_harness {
         my $tst_perl = catfile( curdir(), 'perl' );
         my $verbose = $self->{v} > 1 ? "-v" : "";
         my @run_harness = $self->_run( "$tst_perl harness $verbose $harness" );
-        my $harness_out = $self->_parse_harness( \%inconsistent, $all_ok,
-                                                 @run_harness );
+        my $harness_out = $self->_parse_harness_output( \%inconsistent, $all_ok,
+                                                        @run_harness );
 
         # safeguard against empty results
         $inconsistent{ $_ } ||= 'FAILED' for keys %inconsistent;
@@ -598,7 +614,11 @@ sub make_test_harness {
         $target = $config->has_arg( '-Uuseshrplib' ) ? "static-test" : "test";
     }
 
-    $self->_run_harness_target( $target, $debugging );
+    if ( $self->{hasharness3} ) {
+        $self->_run_harness3_target( $target, $debugging );
+    } else {
+        $self->_run_harness_target( $target, $debugging );
+    }
 }
 
 =item $smoker->_run_harness_target( $target, $extra )
@@ -621,20 +641,21 @@ sub _run_harness_target {
 
     my $tst = $self->_make_fork( $target, $extra );
 
-    while ( <$tst> ) {
-        $self->{v} > 1 and $self->tty( $_ );
+    my $line;
+    while ( $line = <$tst> ) {
+        $self->{v} > 1 and $self->tty( $line );
 
-        /All tests successful/ and push( @failed, $_ ), last;
+        $line =~ /All tests successful/ and push( @failed, $line ), last;
 
-        /Failed Test\s+Stat/ and $seenheader = 1, next;
+        $line =~ /Failed Test\s+Stat/ and $seenheader = 1, next;
         $seenheader or next;
 
-        my( $name, $fail ) = m/$harness_re1/;
+        my( $name, $fail ) = $line =~ m/$harness_re1/;
         if ( $name ) {
             my $dots = '.' x (40 - length $name );
             push @failed, "    $name${dots}FAILED $fail\n";
         } else {
-            ( $fail ) = m/$harness_re2/;
+            ( $fail ) = $line =~ m/$harness_re2/;
             next unless $fail;
             push @failed, " " x 51 . "$fail\n";
         }
@@ -649,6 +670,62 @@ sub _run_harness_target {
     $self->tty( "Archived results...\n" );
 }
 
+=item $smoker->_run_harness3_target( $target, $extra )
+
+The command to run C<make test_harness> differs based on platform, so
+the arguments have to be passed into general routine. C<$target>
+specifies the makefile-target, C<$makeopt> specifies the extra options
+for the make program.
+
+=cut
+
+sub _run_harness3_target {
+    my( $self, $target, $extra ) = @_;
+
+    my $harness3_re = HARNESS3_RE();
+    my $seenheader = 0;
+    my @failed = ( );
+
+    my $tst = $self->_make_fork( $target, $extra );
+
+    my $line;
+    while ( $line = <$tst> ) {
+        $self->{v} > 1 and $self->tty( $line );
+
+        $line =~ /All tests successful/ and push( @failed, $line ), last;
+
+        $line =~ /Test Summary Report/ and $seenheader = 1, next;
+        $seenheader or next;
+    
+        my( $tname ) = $line =~ /^\s*(.+\.t)\s+\(Wstat/;
+        if ( $tname ) {
+            my $dots = '.' x (60 - length $tname);
+            push @failed, "$tname${dots}FAILED\n";
+            next;
+        }
+    
+        my( $failed ) = $line =~ /$harness3_re/x;
+        if ( $failed ) {
+            push @failed, "    $failed\n";
+            next;
+        }
+    
+        my( $parse_error ) = $line =~ /^  Parse errors: (.+)/;
+        if ( $parse_error ) {
+            push @failed, "    $parse_error\n";
+            next;
+        }
+    }
+
+    close $tst or do {
+        my $error = $! || ( $? >> 8);
+        Carp::carp( "\nerror while running harness target '$target': $error" );
+    };
+
+    $self->ttylog( "\n", join( "", @failed ), "\n" );
+    $self->tty( "Archived results...\n" );
+}
+
 sub _run_TEST_target {
     my( $self, $target, $extend ) = @_;
     !$target and Carp::Confess( "No target in _run_TEST_target" );
@@ -720,8 +797,12 @@ to change.
 
 =cut
 
-sub _parse_harness {
+sub _parse_harness_output {
     my( $self, $notok, $all_ok, @lines ) = @_;
+
+    grep m/^Test Summary Report/ => @lines
+        and return $self->_parse_harness3_output( $notok, $all_ok, @lines );
+
     my $harness_re1 = HARNESS_RE1();
     my $harness_re2 = HARNESS_RE2();
 
@@ -745,6 +826,47 @@ sub _parse_harness {
     return $output;
 }
 
+=item $self->_parse_harness3_output( $\%notok, $all_ok, @lines )
+
+Fator out the parsing of the Test::Harness 3 output, as it seems subject
+to change.
+
+=cut
+
+sub _parse_harness3_output {
+    my( $self, $notok, $all_ok, @lines ) = @_;
+
+    my $harness3_re = HARNESS3_RE();
+    my $seenheader = 0;
+    my $output = join "", grep defined $_ => map {
+        my $line = $_;
+
+        my( $tname ) = $line =~ /^\s*(.+\.t)\s+\(Wstat/;
+        my( $failed ) = $line =~ /$harness3_re/x;
+        my( $parse_error ) = $line =~ /^  Parse errors: (.+)/;
+
+        if ( $tname ) {
+            delete $notok->{ $tname };
+            my $dots = '.' x (60 - length $tname);
+            "    $tname${dots}FAILED\n";
+        } elsif ( $failed ) {
+            "        $failed\n";
+        } elsif ( $parse_error ) {
+            "        $parse_error\n";
+        } else {
+            undef;
+        }
+    } grep defined $_ && length $_ => map {
+        $seenheader or $seenheader = $_ =~ /Test Summary Report/;
+        /All tests successful/ && $all_ok++;
+        $self->{v} and $self->tty( $_ );
+        $seenheader ? $_ : '';
+    } @lines;
+
+    $_[2] = $all_ok;
+    return $output;
+}
+
 =item $self->_trasnaform_testnames( @notok )
 
 C<_transform_testnames()> takes a list of testnames, as found by
